"use strict";
var _EventEmitter_listeners;
Object.defineProperty(exports, "__esModule", { value: true });
exports.EventEmitter = void 0;
const tslib_1 = require("tslib");
class EventEmitter {
    constructor() {
        _EventEmitter_listeners.set(this, {});
    }
    _isEventListener(listener) {
        return typeof listener === 'function';
    }
    /**
     * Check if there are listeners for event name
     */
    hasListener(eventName) {
        const listenersBundle = tslib_1.__classPrivateFieldGet(this, _EventEmitter_listeners, "f")[eventName];
        return (listenersBundle && listenersBundle.size > 0);
    }
    /**
     * Emit an event
     * @param eventName Event name
     * @param value Event value that will be passed as an argument to listeners
     */
    emit(eventName, value) {
        if (!this.hasListener(eventName)) {
            return;
        }
        const listenersBundle = tslib_1.__classPrivateFieldGet(this, _EventEmitter_listeners, "f")[eventName];
        if (listenersBundle instanceof Map && listenersBundle.size > 0) {
            listenersBundle.forEach(listener => listener(value));
        }
    }
    /**
     * Add multiple listeners for events
     * @param listenersBundle Map of events to listeners
     * @param ref Reference value that can be used to remove the
     * listeners
     */
    on(listenersBundle, ref) {
        for (const eventName in listenersBundle) {
            if (!listenersBundle.hasOwnProperty(eventName))
                continue;
            const listener = listenersBundle[eventName];
            if (this._isEventListener(listener)) {
                this.addListener(eventName, listener, ref);
            }
        }
    }
    /**
     * Add a listener to an event
     * @param eventName Event name
     * @param listener Listener function
     * @param ref Reference value that can be used to remove this listener,
     * if omitted it will be the listener itself
     */
    addListener(eventName, listener, ref = listener) {
        if (tslib_1.__classPrivateFieldGet(this, _EventEmitter_listeners, "f")[eventName] === undefined) {
            tslib_1.__classPrivateFieldGet(this, _EventEmitter_listeners, "f")[eventName] = new Map();
        }
        tslib_1.__classPrivateFieldGet(this, _EventEmitter_listeners, "f")[eventName]
            .set(ref, listener);
    }
    /**
     * Remove multiple listeners for events
     * @param eventNames Array of event names
     * @param ref Reference value that map to listeners
     */
    off(eventNames, ref) {
        eventNames.forEach(eventName => this.removeListener(eventName, ref));
    }
    // offByRef(ref: EventListenerRef): void {
    //   Object.keys(this.#listeners).forEach(eventName => {
    //     this.removeListener(eventName, ref);
    //   });
    // }
    /**
     * Remove a listener to an event
     * @param eventName Event name
     * @param ref Reference value that was used when adding the listener,
     * this can be the listener itself
     */
    removeListener(eventName, ref) {
        const listenersBundle = tslib_1.__classPrivateFieldGet(this, _EventEmitter_listeners, "f")[eventName];
        if (listenersBundle !== undefined && listenersBundle.has(ref)) {
            listenersBundle.delete(ref);
            if (listenersBundle.size === 0) {
                delete tslib_1.__classPrivateFieldGet(this, _EventEmitter_listeners, "f")[eventName];
            }
        }
    }
}
exports.EventEmitter = EventEmitter;
_EventEmitter_listeners = new WeakMap();
//# sourceMappingURL=event-emitter.js.map